# Generated by Django 5.1.1 on 2024-11-16 13:48

from django.db import migrations, models

import djeocad.models


class Migration(migrations.Migration):

    dependencies = [
        ("djeocad", "0007_entity_rotation_entity_xscale_entity_yscale"),
    ]

    operations = [
        migrations.AlterField(
            model_name="entity",
            name="data",
            field=models.JSONField(default=djeocad.models.get_default_entity_data),
        ),
    ]
