<!doctype html>
<html lang="en">
  <head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <title>Django GeoCAD</title>
    {% block extra-head %}{% endblock extra-head %}
  </head>
  <body>
    <div id="content">
      {% block content %}
        <h1>Hello, world!</h1>
      {% endblock content %}
    </div>
    {% block extra-js %}{% endblock extra-js %}
  </body>
</html>
