{% extends "base.html" %}
{% load leaflet_tags %}

{% block extra-head %}
  {% leaflet_js %}
  {% leaflet_css %}
  <style>
    .leaflet-container {  /* all maps */
      height: 600px;
    }
  </style>
{% endblock extra-head %}

{% block content %}
  {% include "djeocad/htmx/drawing_detail.html" %}
{% endblock content %}
