# Generated by Django 5.1.1 on 2024-11-14 17:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djeocad", "0005_entity_block"),
    ]

    operations = [
        migrations.CreateModel(
            name="EntityData",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("key", models.CharField(max_length=50, verbose_name="Data key")),
                ("value", models.CharField(max_length=100, verbose_name="Data value")),
                (
                    "entity",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="related_data",
                        to="djeocad.entity",
                    ),
                ),
            ],
            options={
                "verbose_name": "Entity Data",
                "verbose_name_plural": "Entity Data",
            },
        ),
    ]
