# Generated by Django 5.1.1 on 2024-12-01 15:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djeocad", "0009_alter_entity_geom"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="layer",
            constraint=models.UniqueConstraint(
                fields=("drawing", "name", "is_block"), name="unique_layer_name"
            ),
        ),
    ]
