# Generated by Django 5.1.1 on 2024-09-16 18:57

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Drawing",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Name of the drawing",
                        max_length=50,
                        verbose_name="Name",
                    ),
                ),
            ],
        ),
    ]
